@@ -19,7 +19,7 @@
  *  SQLite function:
  *       SELECT import_sql(filename);
  *
- *  C function (STANDALONE):
+ *  C function:
  *       int impexp_import_sql(sqlite3 *db, char *filename);
  *
  *       Reads SQL commands from filename and executes them
@@ -30,7 +30,7 @@
  *  SQLite function:
  *       SELECT export_sql(filename, [mode, tablename, ...]);
  *
- *  C function (STANDALONE):
+ *  C function:
  *       int impexp_export_sql(sqlite3 *db, char *filename, int mode, ...);
  *
  *       Writes SQL to filename similar to SQLite's shell
@@ -56,13 +56,15 @@
  *
  *
  *  SQLite function:
- *       SELECT export_csv(filename, prefix1, tablename1, schema1, ...]);
+ *       SELECT export_csv(filename, hdr, prefix1, tablename1, schema1, ...]);
  *
- *  C function (STANDALONE):
- *       int impexp_export_csv(sqlite3 *db, char *prefix1, char *filename,
- *                             char *schema, ...);
+ *  C function:
+ *       int impexp_export_csv(sqlite3 *db, char *filename, int hdr, ...);
+ *                             [char *prefix1, char *tablename1,
+ *                             char *schema1, ...]
  *
- *       Writes entire tables as CSV to provided filename. The
+ *       Writes entire tables as CSV to provided filename. A header
+ *       row is written when the hdr parameter is true. The
  *       rows are optionally introduced with a column made up of
  *       the prefix (non-empty string) for the respective table.
  *       If "schema" is NULL, "sqlite_master" is used, otherwise
@@ -74,18 +76,25 @@
  *          INSERT INTO A VALUES(3,'foo');
  *          CREATE TABLE B(c);
  *          INSERT INTO B VALUES('hello');
- *          SELECT export_csv('out.csv', 'aa', 'A', NULL, 'bb', 'B', NULL);
+ *          SELECT export_csv('out.csv', 0, 'aa', 'A', NULL, 'bb', 'B', NULL);
  *          -- CSV output
  *          "aa",1,2
  *          "aa",3,"foo"
  *          "bb","hello"
+ *          SELECT export_csv('out.csv', 1, 'aa', 'A', NULL, 'bb', 'B', NULL);
+ *          -- CSV output
+ *          "aa","a","b"
+ *          "aa",1,2
+ *          "aa",3,"foo"
+ *          "bb","c"
+ *          "bb","hello"
  *
  *
  *  SQLite function:
  *       SELECT export_xml(filename, appendflag, indent,
  *                         [root, item, tablename, schema]+);
  *
- *  C function (STANDALONE):
+ *  C function:
  *       int impexp_export_xml(sqlite3 *db, char *filename,
  *                             int append, int indent, char *root,
  *                             char *item, char *tablename, char *schema);
@@ -134,12 +143,42 @@
  *       not on column names and root/item tags.
  *
  *
+ *  SQLite function:
+ *       SELECT export_json(filename, sql);
+ *
+ *  C function:
+ *       int impexp_export_json(sqlite3 *db, char *sql,
+ *                              impexp_putc pfunc, void *parg);
+ *
+ *       Executes arbitrary SQL statements and formats
+ *       the result in JavaScript Object Notation (JSON).
+ *       The layout of the result is:
+ *
+ *        object {results, sql}
+ *         results[] object {columns, rows, changes, last_insert_rowid, error}
+ *          columns[]
+ *           object {name, decltype, type }     (sqlite3_column_*)
+ *          rows[][]                            (sqlite3_column_*)
+ *          changes                             (sqlite3_changes)
+ *          last_insert_rowid                   (sqlite3_last_insert_rowid)
+ *          error                               (sqlite3_errmsg)
+ *         sql                                  (SQL text)
+ *
+ *       For each single SQL statement in "sql" an object in the
+ *       "results" array is produced.
+ *
+ *       The function pointer for the output function to
+ *       "impexp_export_json" has a signature compatible
+ *       with fputc(3).
+ *
+ *
  * On Win32 the filename argument may be specified as NULL in order
  * to open a system file dialog for interactive filename selection.
  */
 
 #ifdef STANDALONE
 #include <sqlite3.h>
+#define sqlite3_api_routines void
 #else
 #include <sqlite3ext.h>
 SQLITE_EXTENSION_INIT1
@@ -154,8 +193,19 @@ SQLITE_EXTENSION_INIT1
 #include <windows.h>
 #define strcasecmp  _stricmp
 #define strncasecmp _strnicmp
+#else
+#include <unistd.h>
 #endif
 
+#include "impexp.h"
+
+/* JSON output helper */
+
+typedef struct {
+    impexp_putc pfunc;
+    void *parg;
+} json_pfs;
+
 static const char space_chars[] = " \f\n\r\t\v";
 
 #define ISSPACE(c) ((c) && strchr(space_chars, (c)) != 0)
@@ -737,7 +787,6 @@ done:
     sqlite3_result_int(ctx, sqlite3_changes(db) - changes0);
 }
 
-#ifdef STANDALONE
 int
 impexp_import_sql(sqlite3 *db, char *filename)
 {
@@ -779,7 +828,6 @@ impexp_import_sql(sqlite3 *db, char *filename)
 done:
     return sqlite3_changes(db) - changes0;
 }
-#endif
 
 typedef struct {
     sqlite3 *db;
@@ -840,6 +888,9 @@ table_dump(DUMP_DATA *dd, char **errp, int fmt, const char *query, ...)
 	if (dd->quote_mode >= 0) {
 	    fputc(';', dd->out);
 	}
+	if (dd->quote_mode == -1) {
+	    fputc('\r', dd->out);
+	}
 	if (dd->quote_mode >= -1) {
 	    fputc('\n', dd->out);
 	}
@@ -858,7 +909,7 @@ static char *
 append(char *in, char const *append, char quote)
 {
     int len, i;
-    int nappend = strlen(append);
+    int nappend = append ? strlen(append) : 0;
     int nin = in ? strlen(in) : 0;
     char *tmp;
 
@@ -870,6 +921,8 @@ append(char *in, char const *append, char quote)
 		len++;
 	    }
 	}
+    } else if (!nappend) {
+	return in;
     }
     tmp = (char *) sqlite3_realloc(in, len);
     if (!tmp) {
@@ -890,7 +943,9 @@ append(char *in, char const *append, char quote)
 	*p++ = quote;
 	*p++ = '\0';
     } else {
-	memcpy(in + nin, append, nappend);
+	if (nappend) {
+	    memcpy(in + nin, append, nappend);
+	}
 	in[len - 1] = '\0';
     }
     return in;
@@ -941,7 +996,7 @@ dump_cb(void *udata, int nargs, char **args, char **cols)
     const char *table, *type, *sql;
     DUMP_DATA *dd = (DUMP_DATA *) udata;
 
-    if (nargs != 3) {
+    if (nargs != 3 || args == NULL) {
 	return 1;
     }
     table = args[0];
@@ -1033,9 +1088,10 @@ bailout0:
 	    }
 	}
     }
-    if (strcmp(type, "table") == 0) {
+    if (strcmp(type, "table") == 0 ||
+	(dd->quote_mode < 0 && strcmp(type, "view") == 0)) {
 	sqlite3_stmt *stmt = 0;
-	char *select = 0, *table_info = 0, *tmp = 0;
+	char *select = 0, *hdr = 0, *table_info = 0, *tmp = 0;
 	char buffer[256];
    
 	table_info = append(table_info, "PRAGMA table_info(", 0);
@@ -1048,6 +1104,9 @@ bailout0:
 #endif
 	if (rc != SQLITE_OK || !stmt) {
 bailout1:
+	    if (hdr) {
+		sqlite3_free(hdr);
+	    }
 	    if (select) {
 		sqlite3_free(select);
 	    }
@@ -1075,6 +1134,9 @@ bailout1:
 	    } else {
 		select = append(select, "SELECT ", 0);
 	    }
+	    if (dd->indent) {
+		hdr = append(hdr, select, 0);
+	    }
 	} else {
 	    if (dd->with_schema) {
 		select = append(select, "SELECT 'INSERT INTO ' || ", 0);
@@ -1106,17 +1168,28 @@ bailout1:
 		    select = append(select, " || ',' || ", 0);
 		}
 	    }
-	    rc = sqlite3_finalize(stmt);
+	    rc = sqlite3_reset(stmt);
 	    if (rc != SQLITE_OK) {
 		goto bailout1;
 	    }
 	    select = append(select, "|| ')'", 0);
-#if defined(HAVE_SQLITE3PREPAREV2) && HAVE_SQLITE3PREPAREV2
-	    rc = sqlite3_prepare_v2(dd->db, table_info, -1, &stmt, 0);
-#else
-	    rc = sqlite3_prepare(dd->db, table_info, -1, &stmt, 0);
-#endif
-	    if (rc != SQLITE_OK || !stmt) {
+	}
+	if (dd->quote_mode == -1 && dd->indent) {
+	    rc = sqlite3_step(stmt);
+	    while (rc == SQLITE_ROW) {
+		const char *text = (const char *) sqlite3_column_text(stmt, 1);
+
+		hdr = append(hdr, "quote_csv(", 0);
+		hdr = append(hdr, text, '"');
+		rc = sqlite3_step(stmt);
+		if (rc == SQLITE_ROW) {
+		    hdr = append(hdr, ") || ',' || ", 0);
+		} else {
+		    hdr = append(hdr, ")", 0);
+		}
+	    }
+	    rc = sqlite3_reset(stmt);
+	    if (rc != SQLITE_OK) {
 		goto bailout1;
 	    }
 	}
@@ -1166,13 +1239,13 @@ bailout1:
 	    }
 	    rc = sqlite3_step(stmt);
 	    if (rc == SQLITE_ROW) {
-		if (dd->quote_mode > -1) {
+		if (dd->quote_mode >= -1) {
 		    select = append(select, ") || ',' || ", 0);
 		} else {
 		    select = append(select, " || ", 0);
 		}
 	    } else {
-		if (dd->quote_mode > -1) {
+		if (dd->quote_mode >= -1) {
 		    select = append(select, ") ", 0);
 		} else {
 		    select = append(select, " ", 0);
@@ -1203,6 +1276,12 @@ bailout1:
 	}
 	if (table_info) {
 	    sqlite3_free(table_info);
+	    table_info = 0;
+	}
+	if (hdr) {
+	    rc = table_dump(dd, 0, 0, hdr);
+	    sqlite3_free(hdr);
+	    hdr = 0;
 	}
 	rc = table_dump(dd, 0, 0, select);
 	if (rc == SQLITE_CORRUPT) {
@@ -1211,6 +1290,7 @@ bailout1:
 	}
 	if (select) {
 	    sqlite3_free(select);
+	    select = 0;
 	}
     }
     return 0;
@@ -1384,12 +1464,23 @@ export_csv_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
     if (!filename) {
 	goto done;
     }
+#ifdef _WIN32
+    dd->out = fopen(filename, "wb");
+#else
     dd->out = fopen(filename, "w");
+#endif
     if (!dd->out) {
 	goto done;
     }
     dd->nlines = 0;
-    for (i = 1; i < nargs; i += 3) {
+    if (nargs > 1) {
+	if (sqlite3_value_type(args[1]) != SQLITE_NULL) {
+	    if (sqlite3_value_int(args[1])) {
+		dd->indent = 1;
+	    }
+	}
+    }
+    for (i = 2; i <= nargs - 3; i += 3) {
 	char *schema = 0, *sql;
 
 	dd->where = 0;
@@ -1481,7 +1572,7 @@ export_xml_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
 	goto done;
     }
     dd->nlines = 0;
-    for (i = 3; i < nargs; i += 4) {
+    for (i = 3; i <= nargs - 4; i += 4) {
 	char *root = 0, *schema = 0, *sql;
 
 	if (sqlite3_value_type(args[i]) != SQLITE_NULL) {
@@ -1531,8 +1622,6 @@ done:
     sqlite3_result_int(ctx, dd->nlines);
 }
 
-
-#ifdef STANDALONE
 int
 impexp_export_sql(sqlite3 *db, char *filename, int mode, ...)
 {
@@ -1616,11 +1705,9 @@ impexp_export_sql(sqlite3 *db, char *filename, int mode, ...)
 done:
     return dd->nlines;
 }
-#endif
 
-#ifdef STANDALONE
 int
-impexp_export_csv(sqlite3 *db, char *filename, ...)
+impexp_export_csv(sqlite3 *db, char *filename, int hdr, ...)
 {
     DUMP_DATA dd0, *dd = &dd0;
     va_list ap;
@@ -1638,6 +1725,7 @@ impexp_export_csv(sqlite3 *db, char *filename, ...)
     dd->indent = 0;
     dd->with_schema = 0;
     dd->quote_mode = -1;
+    dd->indent = hdr != 0;
 #ifdef _WIN32
     if (!filename) {
 	OPENFILENAME ofn;
@@ -1657,12 +1745,21 @@ impexp_export_csv(sqlite3 *db, char *filename, ...)
     if (!filename) {
 	goto done;
     }
-    dd->out = fopen(filename, "w");
+#ifdef _WIN32
+    dd->out = fopen(filename, "wb");
+#else
+    if (hdr < 0 && access(filename, W_OK) == 0) {
+	dd->out = fopen(filename, "a");
+	dd->indent = 0;
+    } else {
+	dd->out = fopen(filename, "w");
+    }
+#endif
     if (!dd->out) {
 	goto done;
     }
     dd->nlines = 0;
-    va_start(ap, filename);
+    va_start(ap, hdr);
     prefix = va_arg(ap, char *);
     table = va_arg(ap, char *);
     schema = va_arg(ap, char *);
@@ -1690,9 +1787,7 @@ impexp_export_csv(sqlite3 *db, char *filename, ...)
 done:
     return dd->nlines;
 }
-#endif
 
-#ifdef STANDALONE
 int
 impexp_export_xml(sqlite3 *db, char *filename, int append, int indnt,
 		  char *root, char *item, char *tablename, char *schema)
@@ -1765,16 +1860,376 @@ impexp_export_xml(sqlite3 *db, char *filename, int append, int indnt,
 done:
     return dd->nlines;
 }
+
+static void
+json_pstr(const char *string, json_pfs *pfs)
+{
+    while (*string) {
+	pfs->pfunc(*string, pfs->parg);
+	string++;
+    }
+}
+
+static void
+json_pstrq(const char *string, json_pfs *pfs)
+{
+    impexp_putc pfunc = pfs->pfunc;
+    void *parg = pfs->parg;
+    char buf[64];
+
+    if (!string) {
+	json_pstr("null", pfs);
+	return;
+    }
+    pfunc('"', parg);
+    while (*string) {
+	switch (*string) {
+	case '"':
+	case '\\':
+	    pfunc('\\', parg);
+	    pfunc(*string, parg);
+	    break;
+	case '\b':
+	    pfunc('\\', parg);
+	    pfunc('b', parg);
+	    break;
+	case '\f':
+	    pfunc('\\', parg);
+	    pfunc('f', parg);
+	    break;
+	case '\n':
+	    pfunc('\\', parg);
+	    pfunc('n', parg);
+	    break;
+	case '\r':
+	    pfunc('\\', parg);
+	    pfunc('r', parg);
+	    break;
+	case '\t':
+	    pfunc('\\', parg);
+	    pfunc('t', parg);
+	    break;
+	default:
+	    if ((*string < ' ' && *string > 0) || *string == 0x7f) {
+		sprintf(buf, "\\u%04x", *string);
+		json_pstr(buf, pfs);
+	    } else if (*string < 0) {
+		unsigned char c = string[0];
+		unsigned long uc = 0;
+
+		if (c < 0xc0) {
+		    uc = c;
+		} else if (c < 0xe0) {
+		    if ((string[1] & 0xc0) == 0x80) {
+			uc = ((c & 0x1f) << 6) | (string[1] & 0x3f);
+			++string;
+		    } else {
+			uc = c;
+		    }
+		} else if (c < 0xf0) {
+		    if ((string[1] & 0xc0) == 0x80 &&
+			(string[2] & 0xc0) == 0x80) {
+			uc = ((c & 0x0f) << 12) |
+			     ((string[1] & 0x3f) << 6) | (string[2] & 0x3f);
+			string += 2;
+		    } else {
+			uc = c;
+		    }
+		} else if (c < 0xf8) {
+		    if ((string[1] & 0xc0) == 0x80 &&
+			(string[2] & 0xc0) == 0x80 &&
+			(string[3] & 0xc0) == 0x80) {
+			uc = ((c & 0x03) << 18) |
+			     ((string[1] & 0x3f) << 12) |
+			     ((string[2] & 0x3f) << 6) |
+			     (string[4] & 0x3f);
+			string += 3;
+		    } else {
+			uc = c;
+		    }
+		} else if (c < 0xfc) {
+		    if ((string[1] & 0xc0) == 0x80 &&
+			(string[2] & 0xc0) == 0x80 &&
+			(string[3] & 0xc0) == 0x80 &&
+			(string[4] & 0xc0) == 0x80) {
+			uc = ((c & 0x01) << 24) |
+			     ((string[1] & 0x3f) << 18) |
+			     ((string[2] & 0x3f) << 12) |
+			     ((string[4] & 0x3f) << 6) |
+			     (string[5] & 0x3f);
+			string += 4;
+		    } else {
+			uc = c;
+		    }
+		} else {
+		    /* ignore */
+		    ++string;
+		}
+		if (uc < 0x10000) {
+		    sprintf(buf, "\\u%04lx", uc);
+		} else if (uc < 0x100000) {
+		    uc -= 0x10000;
+
+		    sprintf(buf, "\\u%04lx", 0xd800 | ((uc >> 10) & 0x3ff));
+		    json_pstr(buf, pfs);
+		    sprintf(buf, "\\u%04lx", 0xdc00 | (uc & 0x3ff));
+		} else {
+		    strcpy(buf, "\\ufffd");
+		}
+		json_pstr(buf, pfs);
+	    } else {
+		pfunc(*string, parg);
+	    }
+	    break;
+	}
+	++string;
+    }
+    pfunc('"', parg);
+}
+
+static void
+json_pb64(const unsigned char *blk, int len, json_pfs *pfs)
+{
+    impexp_putc pfunc = pfs->pfunc;
+    void *parg = pfs->parg;
+    int i, reg[5];
+    char buf[16];
+    static const char *b64 =
+	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
+
+    if (!blk) {
+	json_pstr("null", pfs);
+	return;
+    }
+    buf[4] = '\0';
+    pfunc('"', parg);
+    for (i = 0; i < len; i += 3) {
+	reg[1] = reg[2] = reg[3] = reg[4] = 0;
+	reg[0] = blk[i];
+	if (i + 1 < len) {
+	    reg[1] = blk[i + 1];
+	    reg[3] = 1;
+	}
+	if (i + 2 < len) {
+	    reg[2] = blk[i + 2];
+	    reg[4] = 1;
+	}
+	buf[0] = b64[reg[0] >> 2];
+	buf[1] = b64[((reg[0] << 4) & 0x30) | (reg[1] >> 4)];
+	if (reg[3]) {
+	    buf[2] = b64[((reg[1] << 2) & 0x3c) | (reg[2] >> 6)];
+	} else {
+	    buf[2] = '=';
+	}
+	if (reg[4]) {
+	    buf[3] = b64[reg[2] & 0x3f];
+	} else {
+	    buf[3] = '=';
+	}
+	json_pstr(buf, pfs);
+    }
+    pfunc('"', parg);
+}
+
+static int
+json_output(sqlite3 *db, char *sql, impexp_putc pfunc, void *parg)
+{
+    json_pfs pfs0, *pfs = &pfs0;
+    const char *tail = sql;
+    int i, nresults = 0, result = SQLITE_ERROR;
+
+    pfs->pfunc = pfunc;
+    pfs->parg = parg;
+    json_pstr("{sql:", pfs);
+    json_pstrq(sql, pfs);
+    json_pstr(",results:[", pfs);
+    do {
+	sqlite3_stmt *stmt;
+	int firstrow = 1, nrows = 0;
+	char buf[256];
+
+	++nresults;
+	json_pstr(nresults == 1 ? "{" : ",{", pfs);
+	result = sqlite3_prepare(db, tail, -1, &stmt, &tail);
+	if (result != SQLITE_OK) {
+doerr:
+	    if (nrows == 0) {
+		json_pstr("columns:null,rows:null,changes:0,"
+			  "last_insert_rowid:null,", pfs);
+	    }
+	    json_pstr("error:", pfs);
+	    json_pstrq(sqlite3_errmsg(db), pfs);
+	    pfunc('}', parg);
+	    break;
+	}
+	result = sqlite3_step(stmt);
+	while (result == SQLITE_ROW || result == SQLITE_DONE) {
+	    if (firstrow) {
+		for (i = 0; i < sqlite3_column_count(stmt); i++) {
+		    char *type;
+		    json_pstr(i == 0 ? "columns:[" : ",", pfs);
+		    json_pstr("{name:", pfs);
+		    json_pstrq(sqlite3_column_name(stmt, i), pfs);
+		    json_pstr(",decltype:", pfs);
+		    json_pstrq(sqlite3_column_decltype(stmt, i), pfs);
+		    json_pstr(",type:", pfs);
+		    switch (sqlite3_column_type(stmt, i)) {
+		    case SQLITE_INTEGER:
+			type = "integer";
+			break;
+		    case SQLITE_FLOAT:
+			type = "float";
+			break;
+		    case SQLITE_BLOB:
+			type = "blob";
+			break;
+		    case SQLITE_TEXT:
+			type = "text";
+			break;
+		    case SQLITE_NULL:
+			type = "null";
+			break;
+		    default:
+			type = "unknown";
+			break;
+		    }
+		    json_pstrq(type, pfs);
+		    pfunc('}', parg);
+		}
+		if (i) {
+		    pfunc(']', parg);
+		}
+		firstrow = 0;
+	    }
+	    if (result == SQLITE_DONE) {
+		break;
+	    }
+	    ++nrows;
+	    json_pstr(nrows == 1 ? ",rows:[" : ",", pfs);
+	    for (i = 0; i < sqlite3_column_count(stmt); i++) {
+		pfunc(i == 0 ? '[' : ',', parg);
+		switch (sqlite3_column_type(stmt, i)) {
+		case SQLITE_INTEGER:
+		case SQLITE_FLOAT:
+		    json_pstr((char *) sqlite3_column_text(stmt, i), pfs);
+		    break;
+		case SQLITE_BLOB:
+		    json_pb64((unsigned char *) sqlite3_column_blob(stmt, i),
+			      sqlite3_column_bytes(stmt, i), pfs);
+		    break;
+		case SQLITE_TEXT:
+		    json_pstrq((char *) sqlite3_column_text(stmt, i), pfs);
+		    break;
+		case SQLITE_NULL:
+		default:
+		    json_pstr("null", pfs);
+		    break;
+		}
+	    }
+	    json_pstr(i == 0 ? "null]" : "]", pfs);
+	    result = sqlite3_step(stmt);
+	}
+	if (nrows > 0) {
+	    pfunc(']', parg);
+	}
+	result = sqlite3_finalize(stmt);
+	if (result != SQLITE_OK) {
+	    if (nrows > 0) {
+		sprintf(buf,
+#ifdef _WIN32
+			",changes:%d,last_insert_rowid:%I64d",
+#else
+			",changes:%d,last_insert_rowid:%lld",
+#endif
+			sqlite3_changes(db),
+			sqlite3_last_insert_rowid(db));
+		json_pstr(buf, pfs);
+	    }
+	    goto doerr;
+	}
+	if (nrows == 0) {
+	    json_pstr("columns:null,rows:null", pfs);
+	}
+	sprintf(buf,
+#ifdef _WIN32
+		",changes:%d,last_insert_rowid:%I64d",
+#else
+		",changes:%d,last_insert_rowid:%lld",
+#endif
+		sqlite3_changes(db),
+		sqlite3_last_insert_rowid(db));
+	json_pstr(buf, pfs);
+	json_pstr(",error:null}", pfs);
+    } while (tail && *tail);
+    json_pstr("]}", pfs);
+    return result;
+}
+
+static void
+export_json_func(sqlite3_context *ctx, int nargs, sqlite3_value **args)
+{
+    sqlite3 *db = (sqlite3 *) sqlite3_user_data(ctx);
+    int result = -1;
+    char *filename = 0;
+    char *sql = 0;
+    FILE *out = 0;
+#ifdef _WIN32
+    char fnbuf[MAX_PATH];
 #endif
 
-#ifdef STANDALONE
+    if (nargs > 0) {
+	if (sqlite3_value_type(args[0]) != SQLITE_NULL) {
+	    filename = (char *) sqlite3_value_text(args[0]);
+	}
+    }
+#ifdef _WIN32
+    if (!filename) {
+	OPENFILENAME ofn;
+
+	memset(&ofn, 0, sizeof (ofn));
+	memset(fnbuf, 0, sizeof (fnbuf));
+	ofn.lStructSize = sizeof (ofn);
+	ofn.lpstrFile = fnbuf;
+	ofn.nMaxFile = MAX_PATH;
+	ofn.Flags = OFN_HIDEREADONLY | OFN_NOCHANGEDIR | OFN_EXPLORER |
+		    OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
+	if (GetSaveFileName(&ofn)) {
+	    filename = fnbuf;
+	}
+    }
+#endif
+    if (!filename) {
+	goto done;
+    }
+    out = fopen(filename, "w");
+    if (!out) {
+	goto done;
+    }
+    if (nargs > 1) {
+	sql = (char *) sqlite3_value_text(args[1]);
+    }
+    if (sql) {
+	result = json_output(db, sql, (impexp_putc) fputc, out);
+    }
+    fclose(out);
+done:
+    sqlite3_result_int(ctx, result);
+}
+
 int
-impexp_init(sqlite3 *db)
+impexp_export_json(sqlite3 *db, char *sql, impexp_putc pfunc,
+		   void *parg)
+{
+    return json_output(db, sql, pfunc, parg);
+}
+
+#ifdef STANDALONE
+static int
 #else
 int
+#endif
 sqlite3_extension_init(sqlite3 *db, char **errmsg, 
 		       const sqlite3_api_routines *api)
-#endif
 {
     int rc, i;
     static const struct {
@@ -1783,18 +2238,21 @@ sqlite3_extension_init(sqlite3 *db, char **errmsg,
 	int nargs;
 	int textrep;
     } ftab[] = {
-	{ "quote_sql",	quote_func,      -1, SQLITE_UTF8 },
-	{ "import_sql",	import_func,     -1, SQLITE_UTF8 },
-	{ "export_sql",	export_func,     -1, SQLITE_UTF8 },
-	{ "quote_csv",	quote_csv_func,  -1, SQLITE_UTF8 },
-	{ "export_csv",	export_csv_func, -1, SQLITE_UTF8 },
-	{ "indent_xml",	indent_xml_func,  1, SQLITE_UTF8 },
-	{ "quote_xml",	quote_xml_func,  -1, SQLITE_UTF8 },
-	{ "export_xml",	export_xml_func, -1, SQLITE_UTF8 }
+	{ "quote_sql",	 quote_func,       -1, SQLITE_UTF8 },
+	{ "import_sql",	 import_func,      -1, SQLITE_UTF8 },
+	{ "export_sql",	 export_func,      -1, SQLITE_UTF8 },
+	{ "quote_csv",	 quote_csv_func,   -1, SQLITE_UTF8 },
+	{ "export_csv",	 export_csv_func,  -1, SQLITE_UTF8 },
+	{ "indent_xml",	 indent_xml_func,   1, SQLITE_UTF8 },
+	{ "quote_xml",	 quote_xml_func,   -1, SQLITE_UTF8 },
+	{ "export_xml",  export_xml_func,  -1, SQLITE_UTF8 },
+	{ "export_json", export_json_func, -1, SQLITE_UTF8 }
     };
 
 #ifndef STANDALONE
-    SQLITE_EXTENSION_INIT2(api);
+    if (api != NULL) {
+	SQLITE_EXTENSION_INIT2(api);
+    }
 #endif
 
     for (i = 0; i < sizeof (ftab) / sizeof (ftab[0]); i++) {
@@ -1805,9 +2263,14 @@ sqlite3_extension_init(sqlite3 *db, char **errmsg,
 		sqlite3_create_function(db, ftab[i].name, ftab[i].nargs,
 					ftab[i].textrep, 0, 0, 0, 0);
 	    }
-	    return rc;
+	    break;
 	}
     }
     return rc;
 }
 
+int
+impexp_init(sqlite3 *db)
+{
+    return sqlite3_extension_init(db, NULL, NULL);
+}
