@@ -2,26 +2,249 @@
 #
 # Build script for cross compiling and packaging SQLite 3
 # ODBC drivers and tools for Win32 using MinGW and NSIS.
-# Tested on Fedora Core 3/5/8, Debian Etch, RHEL 5.
+# Tested on Fedora Core 3/5/8, Debian Etch, RHEL 5/6.
 #
-# Cross toolchain and NSIS for Linux/i386 can be fetched from
-#  http://www.ch-werner.de/xtools/crossmingw64-0.1-1.i386.rpm
-#  http://www.ch-werner.de/xtools/nsis-2.11-1.i386.rpm
+# Cross toolchain and NSIS for Linux/i386/x86_64 can be fetched from
+#  http://www.ch-werner.de/xtools/crossmingw64-0.2-1.i386.rpm
+#  http://www.ch-werner.de/xtools/crossmingw64-0.2-1.x86_64.rpm
+#  http://www.ch-werner.de/xtools/nsis-2.37-1.i386.rpm
+# or
+#  http://www.ch-werner.de/xtools/crossmingw64-0.2.i386.tar.bz2
+#  http://www.ch-werner.de/xtools/crossmingw64-0.2.x86_64.tar.bz2
+#  http://www.ch-werner.de/xtools/nsis-2.37-1_i386.tar.gz
+
+# Some aspects of the build process can be controlled by shell variables:
+#
+# NO_SQLITE2=1      omit building SQLite 2 and drivers for it
+# SQLITE_DLLS=1     build and package driver with sqlite3.dll
+# SQLITE_DLLS=2     build driver with refs to sqlite3.dll
+#                   driver can use System.Data.SQLite.dll instead
 
 set -e
 
-VER3=3.7.7.1
-VER3X=3070701
+VER2=2.8.17
+VER3=3.7.13
+VER3X=3071300
 
-if test -n "$SQLITE_DLLS" ; then
+nov2=false
+if test -n "$NO_SQLITE2" ; then
+    nov2=true
+    ADD_NSIS="-DWITHOUT_SQLITE2"
+fi
+
+if test -f "$WITH_SEE" ; then
+    export SEEEXT=see
+    ADD_NSIS="-DWITH_SEE=$SEEEXT"
+    if test "$SQLITE_DLLS" = "2" ; then
+	SQLITE_DLLS=1
+    fi
+fi
+
+if test "$SQLITE_DLLS" = "2" ; then
+    # turn on -DSQLITE_DYNLOAD in sqlite3odbc.c
+    export ADD_CFLAGS="-DWITHOUT_SHELL=1 -DWITH_SQLITE_DLLS=2"
+    ADD_NSIS="$ADD_NSIS -DWITHOUT_SQLITE3_EXE"
+elif test -n "$SQLITE_DLLS" ; then
     export ADD_CFLAGS="-DWITHOUT_SHELL=1 -DWITH_SQLITE_DLLS=1"
-    ADD_NSIS="-DWITH_SQLITE_DLLS"
+    export SQLITE3_DLL="-Lsqlite3 -lsqlite3"
+    export SQLITE3_EXE="sqlite3.exe"
+    ADD_NSIS="$ADD_NSIS -DWITH_SQLITE_DLLS"
+else
+    export SQLITE3_A10N_O="sqlite3a10n.o"
+    export SQLITE3_EXE="sqlite3.exe"
 fi
 
 if test -n "$WITH_SOURCES" ; then
     ADD_NSIS="$ADD_NSIS -DWITH_SOURCES"
 fi
 
+echo "===================="
+echo "Preparing sqlite ..."
+echo "===================="
+( $nov2 && echo '*** skipped (NO_SQLITE2)' ) || true
+$nov2 || test -r sqlite-${VER2}.tar.gz || \
+    wget -c http://www.sqlite.org/sqlite-${VER2}.tar.gz
+$nov2 || test -r sqlite-${VER2}.tar.gz || exit 1
+
+$nov2 || rm -f sqlite
+$nov2 || tar xzf sqlite-${VER2}.tar.gz
+$nov2 || ln -sf sqlite-${VER2} sqlite
+
+# enable sqlite_encode_binary et.al.
+$nov2 || patch sqlite/main.mk <<'EOD'
+--- sqlite.orig/main.mk	2005-04-24 00:43:23.000000000 +0200
++++ sqlite/main.mk	2006-03-16 14:29:55.000000000 +0100
+@@ -55,7 +55,7 @@
+ # Object files for the SQLite library.
+ #
+ LIBOBJ = attach.o auth.o btree.o btree_rb.o build.o copy.o date.o delete.o \
+-         expr.o func.o hash.o insert.o \
++         expr.o func.o hash.o insert.o encode.o \
+          main.o opcodes.o os.o pager.o parse.o pragma.o printf.o random.o \
+          select.o table.o tokenize.o trigger.o update.o util.o \
+          vacuum.o vdbe.o vdbeaux.o where.o tclsqlite.o
+EOD
+
+# display encoding
+$nov2 || patch sqlite/src/shell.c <<'EOD'
+--- sqlite.orig/src/shell.c	2005-04-24 00:43:22.000000000 +0200
++++ sqlite/src/shell.c	2006-05-23 08:22:01.000000000 +0200
+@@ -1180,6 +1180,7 @@
+   "   -separator 'x'       set output field separator (|)\n"
+   "   -nullvalue 'text'    set text string for NULL values\n"
+   "   -version             show SQLite version\n"
++  "   -encoding            show SQLite encoding\n"
+   "   -help                show this text, also show dot-commands\n"
+ ;
+ static void usage(int showDetail){
+@@ -1297,7 +1298,10 @@
+     }else if( strcmp(z,"-echo")==0 ){
+       data.echoOn = 1;
+     }else if( strcmp(z,"-version")==0 ){
+-      printf("%s\n", sqlite_version);
++      printf("%s\n", sqlite_libversion());
++      return 1;
++    }else if( strcmp(z,"-encoding")==0 ){
++      printf("%s\n", sqlite_libencoding());
+       return 1;
+     }else if( strcmp(z,"-help")==0 ){
+       usage(1);
+@@ -1330,9 +1334,9 @@
+       char *zHome;
+       char *zHistory = 0;
+       printf(
+-        "SQLite version %s\n"
++        "SQLite version %s encoding %s\n"
+         "Enter \".help\" for instructions\n",
+-        sqlite_version
++        sqlite_libversion(), sqlite_libencoding()
+       );
+       zHome = find_home_dir();
+       if( zHome && (zHistory = malloc(strlen(zHome)+20))!=0 ){
+EOD
+
+# use open file dialog when no database name given
+# need to link with -lcomdlg32 when enabled
+true || patch sqlite/src/shell.c <<'EOD'
+--- sqlite.orig/src/shell.c        2006-07-23 11:18:13.000000000 +0200
++++ sqlite/src/shell.c     2006-07-23 11:30:26.000000000 +0200
+@@ -20,6 +20,10 @@
+ #include "sqlite.h"
+ #include <ctype.h>
+ 
++#if defined(_WIN32) && defined(DRIVER_VER_INFO)
++# include <windows.h>
++#endif
++
+ #if !defined(_WIN32) && !defined(WIN32) && !defined(__MACOS__)
+ # include <signal.h>
+ # include <pwd.h>
+@@ -1246,6 +1250,17 @@
+   if( i<argc ){
+     data.zDbFilename = argv[i++];
+   }else{
++#if defined(_WIN32) && defined(DRIVER_VER_INFO)
++    static OPENFILENAME ofn;
++    static char zDbFn[1024];
++    ofn.lStructSize = sizeof(ofn);
++    ofn.lpstrFile = (LPTSTR) zDbFn;
++    ofn.nMaxFile = sizeof(zDbFn);
++    ofn.Flags = OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_NOCHANGEDIR;
++    if( GetOpenFileName(&ofn) ){
++      data.zDbFilename = zDbFn;
++    } else
++#endif
+     data.zDbFilename = ":memory:";
+   }
+   if( i<argc ){
+EOD
+
+# same but new module libshell.c
+$nov2 || patch sqlite/main.mk <<'EOD'
+--- sqlite.orig/main.mk        2007-01-10 19:30:52.000000000 +0100
++++ sqlite/main.mk     2007-01-10 19:33:39.000000000 +0100
+@@ -54,7 +54,7 @@
+ 
+ # Object files for the SQLite library.
+ #
+-LIBOBJ = attach.o auth.o btree.o btree_rb.o build.o copy.o date.o delete.o \
++LIBOBJ += attach.o auth.o btree.o btree_rb.o build.o copy.o date.o delete.o \
+          expr.o func.o hash.o insert.o encode.o \
+          main.o opcodes.o os.o pager.o parse.o pragma.o printf.o random.o \
+          select.o table.o tokenize.o trigger.o update.o util.o \
+EOD
+$nov2 || cp -p sqlite/src/shell.c sqlite/src/libshell.c
+$nov2 || patch sqlite/src/libshell.c <<'EOD'
+--- sqlite.orig/src/libshell.c  2007-01-10 19:13:01.000000000 +0100
++++ sqlite/src/libshell.c  2007-01-10 19:25:56.000000000 +0100
+@@ -20,6 +20,10 @@
+ #include "sqlite.h"
+ #include <ctype.h>
+ 
++#ifdef _WIN32
++# include <windows.h>
++#endif
++
+ #if !defined(_WIN32) && !defined(WIN32) && !defined(__MACOS__)
+ # include <signal.h>
+ # include <pwd.h>
+@@ -1205,7 +1209,7 @@
+   strcpy(continuePrompt,"   ...> ");
+ }
+ 
+-int main(int argc, char **argv){
++int sqlite_main(int argc, char **argv){
+   char *zErrMsg = 0;
+   struct callback_data data;
+   const char *zInitFile = 0;
+@@ -1246,6 +1250,17 @@
+   if( i<argc ){
+     data.zDbFilename = argv[i++];
+   }else{
++#if defined(_WIN32) && !defined(__TINYC__)
++    static OPENFILENAME ofn;
++    static char zDbFn[1024];
++    ofn.lStructSize = sizeof(ofn);
++    ofn.lpstrFile = (LPTSTR) zDbFn;
++    ofn.nMaxFile = sizeof(zDbFn);
++    ofn.Flags = OFN_PATHMUSTEXIST | OFN_EXPLORER | OFN_NOCHANGEDIR;
++    if( GetOpenFileName(&ofn) ){
++      data.zDbFilename = zDbFn;
++    } else
++#endif
+     data.zDbFilename = ":memory:";
+   }
+   if( i<argc ){
+EOD
+
+$nov2 || rm -f sqlite/src/minshell.c
+$nov2 || touch sqlite/src/minshell.c
+$nov2 || patch sqlite/src/minshell.c <<'EOD'
+--- sqlite.orig/src/minshell.c  2007-01-10 18:46:47.000000000 +0100
++++ sqlite/src/minshell.c  2007-01-10 18:46:47.000000000 +0100
+@@ -0,0 +1,20 @@
++/*
++** 2001 September 15
++**
++** The author disclaims copyright to this source code.  In place of
++** a legal notice, here is a blessing:
++**
++**    May you do good and not evil.
++**    May you find forgiveness for yourself and forgive others.
++**    May you share freely, never taking more than you give.
++**
++*************************************************************************
++** This file contains code to implement the "sqlite" command line
++** utility for accessing SQLite databases.
++*/
++
++int sqlite_main(int argc, char **argv);
++
++int main(int argc, char **argv){
++  return sqlite_main(argc, argv);
++}
+EOD
+
 echo "====================="
 echo "Preparing sqlite3 ..."
 echo "====================="
@@ -226,7 +449,9 @@ test "$VER3" != "3.6.15" -a "$VER3" != "3.6.16" -a "$VER3" != "3.6.17" \
   -a "$VER3" != "3.7.1" -a "$VER3" != "3.7.2" -a "$VER3" != "3.7.3" \
   -a "$VER3" != "3.7.4" -a "$VER3" != "3.7.5" -a "$VER3" != "3.7.6" \
   -a "$VER3" != "3.7.6.1" -a "$VER3" != "3.7.6.2" -a "$VER3" != "3.7.6.3" \
-  -a "$VER3" != "3.7.7" -a "$VER3" != "3.7.7.1" \
+  -a "$VER3" != "3.7.7" -a "$VER3" != "3.7.7.1" -a "$VER3" != "3.7.8" \
+  -a "$VER3" != "3.7.9" -a "$VER3" != "3.7.10" -a "$VER3" != "3.7.11" \
+  -a "$VER3" != "3.7.12" -a "$VER3" != "3.7.12.1" -a "$VER3" != "3.7.13" \
   && patch -d sqlite3 -p1 <<'EOD'
 diff -u sqlite3.orig/src/build.c sqlite3/src/build.c
 --- sqlite3.orig/src/build.c	2007-01-09 14:53:04.000000000 +0100
@@ -291,7 +516,7 @@ diff -u sqlite3.orig/src/tclsqlite.c sqlite3/src/tclsqlite.c
 +++ sqlite3/src/tclsqlite.c	2007-04-10 07:47:49.000000000 +0200
 @@ -14,6 +14,7 @@
  **
- ** $Id: mingw64-cross-build.sh,v 1.19 2011/07/04 05:51:54 chw Exp chw $
+ ** $Id: mingw64-cross-build.sh,v 1.28 2012/06/24 09:36:50 chw Exp chw $
  */
 +#ifndef NO_TCL     /* Omit this whole file if TCL is unavailable */
  #include "tcl.h"
@@ -443,7 +668,9 @@ test "$VER3" != "3.6.21" -a "$VER3" != "3.6.22" -a "$VER3" != "3.6.23" \
   -a "$VER3" != "3.7.1" -a "$VER3" != "3.7.2" -a "$VER3" != "3.7.3" \
   -a "$VER3" != "3.7.4" -a "$VER3" != "3.7.5" -a "$VER3" != "3.7.6" \
   -a "$VER3" != "3.7.6.1" -a "$VER3" != "3.7.6.2" -a "$VER3" != "3.7.6.3" \
-  -a "$VER3" != "3.7.7" -a "$VER3" != "3.7.7.1" \
+  -a "$VER3" != "3.7.7" -a "$VER3" != "3.7.7.1" -a "$VER3" != "3.7.8" \
+  -a "$VER3" != "3.7.9" -a "$VER3" != "3.7.10" -a "$VER3" != "3.7.11" \
+  -a "$VER3" != "3.7.12" -a "$VER3" != "3.7.12.1" -a "$VER3" != "3.7.13" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3.c 2008-02-02 17:24:34.000000000 +0100
 +++ sqlite3/ext/fts3/fts3.c      2008-03-16 11:29:02.000000000 +0100
@@ -580,6 +807,11 @@ patch -d sqlite3 -p1 <<'EOD'
  #include "fts3_tokenizer.h"
  
  typedef struct simple_tokenizer {
+EOD
+test "$VER3" != "3.7.8" -a "$VER3" != "3.7.9" -a "$VER3" != "3.7.10" \
+  -a "$VER3" != "3.7.11" -a "$VER3" != "3.7.12" -a "$VER3" != "3.7.12.1" \
+  -a "$VER3" != "3.7.13" \
+  && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3_hash.c    2007-11-24 01:41:52.000000000 +0100
 +++ sqlite3/ext/fts3/fts3_hash.c 2008-03-16 11:39:57.000000000 +0100
 @@ -29,6 +29,11 @@
@@ -936,7 +1168,9 @@ EOD
 test "$VER3" = "3.7.3" -o "$VER3" = "3.7.4" -o "$VER3" = "3.7.5" \
   -o "$VER3" = "3.7.6" \
   -o "$VER3" = "3.7.6.1" -o "$VER3" = "3.7.6.2" -o "$VER3" = "3.7.6.3" \
-  -o "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
+  -o "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
+  -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
+  -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/rtree/rtree.c	2010-10-16 10:53:54.000000000 +0200
 +++ sqlite3/ext/rtree/rtree.c	2010-10-16 11:12:32.000000000 +0200
@@ -962,7 +1196,9 @@ EOD
 
 # patch: .read shell command
 test "$VER3" = "3.7.6.1" -o "$VER3" = "3.7.6.2" -o "$VER3" = "3.7.6.3" \
-  -o "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
+  -o "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
+  -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
+  -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/src/shell.c	2011-05-19 15:34:57.000000000 +0200
 +++ sqlite3/src/shell.c	2011-06-09 13:36:13.000000000 +0200
@@ -977,7 +1213,9 @@ test "$VER3" = "3.7.6.1" -o "$VER3" = "3.7.6.2" -o "$VER3" = "3.7.6.3" \
 EOD
 
 # patch: FTS3 for 3.7.7 plus missing APIs in sqlite3ext.h/loadext.c
-test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
+test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
+  -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
+  -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
   && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3_aux.c	2011-06-24 09:06:08.000000000 +0200
 +++ sqlite3/ext/fts3/fts3_aux.c	2011-06-25 06:44:08.000000000 +0200
@@ -992,6 +1230,9 @@ test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
  #include <string.h>
  #include <assert.h>
  
+EOD
+test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
+  && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3.c	2011-06-24 09:06:08.000000000 +0200
 +++ sqlite3/ext/fts3/fts3.c	2011-06-25 06:48:49.000000000 +0200
 @@ -295,10 +295,6 @@
@@ -1014,6 +1255,38 @@ test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
  int sqlite3_extension_init(
    sqlite3 *db, 
    char **pzErrMsg,
+EOD
+test "$VER3" = "3.7.8" -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" \
+  -o "$VER3" = "3.7.11" -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" \
+  -o "$VER3" = "3.7.13" \
+  && patch -d sqlite3 -p1 <<'EOD'
+--- sqlite3.orig/ext/fts3/fts3.c	2011-09-19 20:46:52.000000000 +0200
++++ sqlite3/ext/fts3/fts3.c	2011-09-20 09:47:40.000000000 +0200
+@@ -295,10 +295,6 @@
+ #include "fts3Int.h"
+ #if !defined(SQLITE_CORE) || defined(SQLITE_ENABLE_FTS3)
+ 
+-#if defined(SQLITE_ENABLE_FTS3) && !defined(SQLITE_CORE)
+-# define SQLITE_CORE 1
+-#endif
+-
+ #include <assert.h>
+ #include <stdlib.h>
+ #include <stddef.h>
+@@ -4826,7 +4822,7 @@
+   }
+ }
+ 
+-#if !SQLITE_CORE
++#ifndef SQLITE_CORE
+ /*
+ ** Initialize API pointer table, if required.
+ */
+EOD
+test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" -o "$VER3" = "3.7.8" \
+  -o "$VER3" = "3.7.9" -o "$VER3" = "3.7.10" -o "$VER3" = "3.7.11" \
+  -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" -o "$VER3" = "3.7.13" \
+  && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3_expr.c	2011-06-24 09:06:08.000000000 +0200
 +++ sqlite3/ext/fts3/fts3_expr.c	2011-06-25 06:47:00.000000000 +0200
 @@ -18,6 +18,11 @@
@@ -1042,6 +1315,9 @@ test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
  #include <string.h>
  #include <assert.h>
  
+EOD
+test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
+  && patch -d sqlite3 -p1 <<'EOD'
 --- sqlite3.orig/ext/fts3/fts3_tokenizer.c	2011-06-24 09:06:08.000000000 +0200
 +++ sqlite3/ext/fts3/fts3_tokenizer.c	2011-06-25 06:50:19.000000000 +0200
 @@ -25,7 +25,7 @@
@@ -1127,22 +1403,92 @@ test "$VER3" = "3.7.7" -o "$VER3" = "3.7.7.1" \
  
  /*
 EOD
+test "$VER3" = "3.7.11" -o "$VER3" = "3.7.12" -o "$VER3" = "3.7.12.1" \
+  -o "$VER3" = "3.7.13" \
+  && patch -d sqlite3 -p1 <<'EOD'
+--- sqlite3.orig/src/sqlite3ext.h     2012-03-22 20:13:33.000000000 +0100
++++ sqlite3/src/sqlite3ext.h  2012-03-22 20:13:57.000000000 +0100
+@@ -236,6 +236,7 @@
+   int (*blob_reopen)(sqlite3_blob*,sqlite3_int64);
+   int (*vtab_config)(sqlite3*,int op,...);
+   int (*vtab_on_conflict)(sqlite3*);
++  int (*stricmp)(const char*,const char*);
+ };
+ 
+ /*
+@@ -439,6 +440,7 @@
+ #define sqlite3_blob_reopen            sqlite3_api->blob_reopen
+ #define sqlite3_vtab_config            sqlite3_api->vtab_config
+ #define sqlite3_vtab_on_conflict       sqlite3_api->vtab_on_conflict
++#define sqlite3_stricmp                sqlite3_api->stricmp
+ #endif /* SQLITE_CORE */
+ 
+ #define SQLITE_EXTENSION_INIT1     const sqlite3_api_routines *sqlite3_api = 0;
+--- sqlite3.orig/src/loadext.c        2012-03-20 15:20:13.000000000 +0100
++++ sqlite3/src/loadext.c     2012-03-22 20:16:24.000000000 +0100
+@@ -378,6 +378,7 @@
+   sqlite3_blob_reopen,
+   sqlite3_vtab_config,
+   sqlite3_vtab_on_conflict,
++  sqlite3_stricmp,
+ };
+ 
+ /*
+EOD
 
 echo "========================"
 echo "Cleanup before build ..."
 echo "========================"
 make -f Makefile.mingw64-cross clean
+$notv2 || make -C sqlite -f ../mf-sqlite.mingw64-cross clean
 make -C sqlite3 -f ../mf-sqlite3.mingw64-cross clean
 make -C sqlite3 -f ../mf-sqlite3fts.mingw64-cross clean
 make -C sqlite3 -f ../mf-sqlite3rtree.mingw64-cross clean
 make -f mf-sqlite3extfunc.mingw64-cross clean
 
+echo "============================="
+echo "Building SQLite 2 ... ISO8859"
+echo "============================="
+( $nov2 && echo '*** skipped (NO_SQLITE2)' ) || true
+$nov2 || make -C sqlite -f ../mf-sqlite.mingw64-cross all
+if test -n "$SQLITE_DLLS" ; then
+    $nov2 || make -C sqlite -f ../mf-sqlite.mingw64-cross sqlite.dll
+fi
+
+echo "=========================="
+echo "Building SQLite 2 ... UTF8"
+echo "=========================="
+( $nov2 && echo '*** skipped (NO_SQLITE2)' ) || true
+$nov2 || make -C sqlite -f ../mf-sqlite.mingw64-cross clean
+$nov2 || make -C sqlite -f ../mf-sqlite.mingw64-cross ENCODING=UTF8 all
+if test -n "$SQLITE_DLLS" ; then
+    $nov2 || \
+       make -C sqlite -f ../mf-sqlite.mingw64-cross ENCODING=UTF8 sqliteu.dll
+fi
+
 echo "====================="
 echo "Building SQLite 3 ..."
 echo "====================="
 make -C sqlite3 -f ../mf-sqlite3.mingw64-cross all
 test -r sqlite3/tool/mksqlite3c.tcl && \
   make -C sqlite3 -f ../mf-sqlite3.mingw64-cross sqlite3.c
+if test -r sqlite3/sqlite3.c -a -f "$WITH_SEE" ; then
+    cat sqlite3/sqlite3.c "$WITH_SEE" > sqlite3.c
+    ADD_CFLAGS="$ADD_CFLAGS -DSQLITE_HAS_CODEC=1"
+    ADD_CFLAGS="$ADD_CFLAGS -DSQLITE_ACTIVATION_KEY=\\\"$SEE_KEY\\\""
+    ADD_CFLAGS="$ADD_CFLAGS -DSEEEXT=\\\"$SEEEXT\\\""
+    ADD_CFLAGS="$ADD_CFLAGS -DSQLITE_API=static -DWIN32=1 -DNDEBUG=1 -DNO_TCL"
+    ADD_CFLAGS="$ADD_CFLAGS -DTHREADSAFE=1 -DSQLITE_OMIT_EXPLAIN=1"
+    ADD_CFLAGS="$ADD_CFLAGS -DSQLITE_DLL=1 -DSQLITE_THREADSAFE=1"
+    ADD_CFLAGS="$ADD_CFLAGS -DSQLITE_OS_WIN=1 -DSQLITE_ASCII=1"
+    ADD_CFLAGS="$ADD_CFLAGS -DSQLITE_SOUNDEX=1"
+    ADD_CFLAGS="$ADD_CFLAGS -DSQLITE_ENABLE_COLUMN_METADATA=1"
+    ADD_CFLAGS="$ADD_CFLAGS -DWITHOUT_SHELL=1"
+    export ADD_CFLAGS
+    ADD_NSIS="$ADD_NSIS -DWITHOUT_SQLITE3_EXE"
+    unset SQLITE3_A10N_O
+    unset SQLITE3_EXE
+fi
 if test -n "$SQLITE_DLLS" ; then
     make -C sqlite3 -f ../mf-sqlite3.mingw64-cross sqlite3.dll
 fi
@@ -1150,8 +1496,12 @@ fi
 echo "==============================="
 echo "Building ODBC drivers and utils"
 echo "==============================="
-make -f Makefile.mingw64-cross
-make -f Makefile.mingw64-cross sqlite3odbcnw.dll
+if $nov2 ; then
+    make -f Makefile.mingw64-cross all_no2
+else
+    make -f Makefile.mingw64-cross
+fi
+make -f Makefile.mingw64-cross sqlite3odbc${SEEEXT}nw.dll
 
 echo "==================================="
 echo "Building SQLite3 FTS extensions ..."
@@ -1170,9 +1520,17 @@ echo "Building SQLite3 extension functions ..."
 echo "========================================"
 make -f mf-sqlite3extfunc.mingw64-cross clean all
 
+echo "========================="
+echo "Building drivers ... UTF8"
+echo "========================="
+( $nov2 && echo '*** skipped (NO_SQLITE2)' ) || true
+$nov2 || make -f Makefile.mingw64-cross sqliteodbcu.dll sqliteu.exe
+
 echo "======================="
 echo "Cleanup after build ..."
 echo "======================="
+$nov2 || make -C sqlite -f ../mf-sqlite.mingw64-cross clean
+$nov2 || rm -f sqlite/sqlite.exe
 make -C sqlite3 -f ../mf-sqlite3.mingw64-cross clean
 make -C sqlite3 -f ../mf-sqlite3fts.mingw64-cross clean
 make -C sqlite3 -f ../mf-sqlite3rtree.mingw64-cross clean
